@@ -1,6 +1,6 @@
 """Arbitrary precision arithmetic."""
 
-__version__ = '5.0.0'
+__version__ = '5.1.0'
 
 try:
     from ._rithm import (Fraction,
@@ -16,6 +16,9 @@ except ImportError:
 
 
     class Int:
+        def bit_length(self) -> 'Int':
+            return Int(self._value.bit_length())
+
         def gcd(self, other: 'Int') -> 'Int':
             return Int(_gcd(self._value, other._value))
 
@@ -38,6 +41,11 @@ except ImportError:
                     if isinstance(other, Int)
                     else NotImplemented)
 
+        def __and__(self, other: 'Int') -> 'Int':
+            return (Int(self._value & other._value)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
         def __bool__(self) -> bool:
             return bool(self._value)
 
@@ -116,6 +124,11 @@ except ImportError:
         def __neg__(self) -> 'Int':
             return Int(-self._value)
 
+        def __or__(self, other: 'Int') -> 'Int':
+            return (Int(self._value | other._value)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
         def __pow__(self,
                     exponent: 'Int',
                     divisor: _Optional['Int'] = None
@@ -158,6 +171,11 @@ except ImportError:
         def __trunc__(self) -> 'Int':
             return self
 
+        def __xor__(self, other: 'Int') -> 'Int':
+            return (Int(self._value ^ other._value)
+                    if isinstance(other, Int)
+                    else NotImplemented)
+
 
     _ONE = Int('1')
     _ZERO = Int()
