@@ -3,16 +3,19 @@ use std::convert::{FloatToInt, TryFrom};
 use std::fmt::{Debug, Display, Formatter};
 use std::iter::Peekable;
 use std::mem::size_of;
-use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Not, Rem, Sub, SubAssign};
+use std::ops::{
+    Add, AddAssign, BitAnd, BitAndAssign, BitOr, BitOrAssign, BitXor, BitXorAssign, Div, DivAssign,
+    Mul, MulAssign, Neg, Not, Rem, Sub, SubAssign,
+};
 use std::str::Chars;
 
 use crate::digits::*;
 use crate::traits::{
-    Abs, AssigningShiftingLeftMonoid, BitwiseNegatableUnaryAlgebra, CheckedDiv, CheckedDivAsF32,
-    CheckedDivAsF64, CheckedDivEuclid, CheckedDivRem, CheckedDivRemEuclid, CheckedPow,
-    CheckedPowRemEuclid, CheckedRem, CheckedRemEuclid, CheckedRemEuclidInv, CheckedShl, CheckedShr,
-    DivEuclid, DivRem, DivRemEuclid, DoublePrecisionOf, Float, FromStrRadix, Gcd, Oppose,
-    OppositionOf, Oppositive, Pow, RemEuclid, Unitary, Zeroable,
+    Abs, AssigningShiftingLeftMonoid, BitLength, BitwiseNegatableUnaryAlgebra, CheckedDiv,
+    CheckedDivAsF32, CheckedDivAsF64, CheckedDivEuclid, CheckedDivRem, CheckedDivRemEuclid,
+    CheckedPow, CheckedPowRemEuclid, CheckedRem, CheckedRemEuclid, CheckedRemEuclidInv, CheckedShl,
+    CheckedShr, DivEuclid, DivRem, DivRemEuclid, DoublePrecisionOf, Float, FromStrRadix, Gcd,
+    Oppose, OppositionOf, Oppositive, Pow, RemEuclid, Unitary, Zeroable,
 };
 use crate::utils;
 
@@ -408,6 +411,107 @@ impl<Digit: AdditiveDigit, const SEPARATOR: char, const SHIFT: usize> AddAssign
     }
 }
 
+impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitAnd
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Self;
+
+    fn bitand(self, other: Self) -> Self::Output {
+        let (sign, digits) = if self.digits.len() > other.digits.len() {
+            bitwise_and::<Digit, SHIFT>(self.digits, self.sign, other.digits, other.sign)
+        } else {
+            bitwise_and::<Digit, SHIFT>(other.digits, other.sign, self.digits, self.sign)
+        };
+        Self { sign, digits }
+    }
+}
+
+impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitAndAssign
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    fn bitand_assign(&mut self, other: Self) {
+        (self.sign, self.digits) = if self.digits.len() > other.digits.len() {
+            bitwise_and::<Digit, SHIFT>(self.digits.clone(), self.sign, other.digits, other.sign)
+        } else {
+            bitwise_and::<Digit, SHIFT>(other.digits, other.sign, self.digits.clone(), self.sign)
+        };
+    }
+}
+
+impl<
+        Digit: BitLength<Output = usize> + BinaryDigit + MultiplicativeDigit + Oppose + TryFrom<usize>,
+        const SEPARATOR: char,
+        const SHIFT: usize,
+    > BitLength for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Self;
+
+    fn bit_length(self) -> Self::Output {
+        if self.digits.len() <= usize::MAX / SHIFT {
+            Self::from(
+                (self.digits.len() - 1) * SHIFT + self.digits[self.digits.len() - 1].bit_length(),
+            )
+        } else {
+            Self::from(self.digits.len() - 1) * Self::from(SHIFT)
+                + Self::from(self.digits[self.digits.len() - 1].bit_length())
+        }
+    }
+}
+
+impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitOr
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Self;
+
+    fn bitor(self, other: Self) -> Self::Output {
+        let (sign, digits) = if self.digits.len() > other.digits.len() {
+            bitwise_or::<Digit, SHIFT>(self.digits, self.sign, other.digits, other.sign)
+        } else {
+            bitwise_or::<Digit, SHIFT>(other.digits, other.sign, self.digits, self.sign)
+        };
+        Self { sign, digits }
+    }
+}
+
+impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitOrAssign
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    fn bitor_assign(&mut self, other: Self) {
+        (self.sign, self.digits) = if self.digits.len() > other.digits.len() {
+            bitwise_or::<Digit, SHIFT>(self.digits.clone(), self.sign, other.digits, other.sign)
+        } else {
+            bitwise_or::<Digit, SHIFT>(other.digits, other.sign, self.digits.clone(), self.sign)
+        };
+    }
+}
+
+impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitXor
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    type Output = Self;
+
+    fn bitxor(self, other: Self) -> Self::Output {
+        let (sign, digits) = if self.digits.len() > other.digits.len() {
+            bitwise_xor::<Digit, SHIFT>(self.digits, self.sign, other.digits, other.sign)
+        } else {
+            bitwise_xor::<Digit, SHIFT>(other.digits, other.sign, self.digits, self.sign)
+        };
+        Self { sign, digits }
+    }
+}
+
+impl<Digit: BinaryDigit, const SEPARATOR: char, const SHIFT: usize> BitXorAssign
+    for BigInt<Digit, SEPARATOR, SHIFT>
+{
+    fn bitxor_assign(&mut self, other: Self) {
+        (self.sign, self.digits) = if self.digits.len() > other.digits.len() {
+            bitwise_xor::<Digit, SHIFT>(self.digits.clone(), self.sign, other.digits, other.sign)
+        } else {
+            bitwise_xor::<Digit, SHIFT>(other.digits, other.sign, self.digits.clone(), self.sign)
+        };
+    }
+}
+
 impl<Digit: DivisibleDigit, const SEPARATOR: char, const SHIFT: usize> CheckedDiv
     for BigInt<Digit, SEPARATOR, SHIFT>
 {
@@ -830,43 +934,42 @@ impl<Digit: DigitConvertibleFromFloat, const SEPARATOR: char, const SHIFT: usize
     }
 }
 
-impl<SourceDigit, TargetDigit, const SEPARATOR: char, const SHIFT: usize> From<SourceDigit>
-    for BigInt<TargetDigit, SEPARATOR, SHIFT>
+impl<Source, Digit, const SEPARATOR: char, const SHIFT: usize> From<Source>
+    for BigInt<Digit, SEPARATOR, SHIFT>
 where
-    SourceDigit: BinaryDigit + Oppose + TryFrom<OppositionOf<SourceDigit>>,
-    TargetDigit: BinaryDigit + Oppose + TryFrom<SourceDigit>,
-    OppositionOf<SourceDigit>: TryFrom<SourceDigit>,
+    Source: BinaryDigit + Oppose + TryFrom<OppositionOf<Source>>,
+    Digit: BinaryDigit + Oppose + TryFrom<Source>,
+    OppositionOf<Source>: TryFrom<Source>,
 {
-    fn from(value: SourceDigit) -> Self {
-        debug_assert!(is_valid_shift::<TargetDigit, SHIFT>());
+    fn from(value: Source) -> Self {
+        debug_assert!(is_valid_shift::<Digit, SHIFT>());
         if value.is_zero() {
             Self::zero()
-        } else if size_of::<SourceDigit>() < size_of::<TargetDigit>()
-            || (size_of::<SourceDigit>() == size_of::<TargetDigit>()
-                && utils::is_signed::<SourceDigit>()
-                && utils::is_unsigned::<TargetDigit>())
+        } else if size_of::<Source>() < size_of::<Digit>()
+            || (size_of::<Source>() == size_of::<Digit>()
+                && utils::is_signed::<Source>()
+                && utils::is_unsigned::<Digit>())
         {
             let mut sign = Sign::one();
-            let mut value = if utils::is_signed::<SourceDigit>() {
-                let value =
-                    unsafe { OppositionOf::<SourceDigit>::try_from(value).unwrap_unchecked() };
+            let mut value = if utils::is_signed::<Source>() {
+                let value = unsafe { OppositionOf::<Source>::try_from(value).unwrap_unchecked() };
                 if value.is_negative() {
                     sign = -sign;
                     unsafe {
-                        TargetDigit::try_from(SourceDigit::try_from(-value).unwrap_unchecked())
+                        Digit::try_from(Source::try_from(-value).unwrap_unchecked())
                             .unwrap_unchecked()
                     }
                 } else {
                     unsafe {
-                        TargetDigit::try_from(SourceDigit::try_from(value).unwrap_unchecked())
+                        Digit::try_from(Source::try_from(value).unwrap_unchecked())
                             .unwrap_unchecked()
                     }
                 }
             } else {
-                unsafe { TargetDigit::try_from(value).unwrap_unchecked() }
+                unsafe { Digit::try_from(value).unwrap_unchecked() }
             };
-            let mut digits = Vec::<TargetDigit>::new();
-            let digit_mask = to_digit_mask::<TargetDigit>(SHIFT);
+            let mut digits = Vec::<Digit>::new();
+            let digit_mask = to_digit_mask::<Digit>(SHIFT);
             while !value.is_zero() {
                 digits.push(value & digit_mask);
                 value >>= SHIFT;
@@ -874,23 +977,21 @@ where
             Self { sign, digits }
         } else {
             let mut sign = Sign::one();
-            let mut value = if utils::is_signed::<SourceDigit>() {
-                let value =
-                    unsafe { OppositionOf::<SourceDigit>::try_from(value).unwrap_unchecked() };
+            let mut value = if utils::is_signed::<Source>() {
+                let value = unsafe { OppositionOf::<Source>::try_from(value).unwrap_unchecked() };
                 if value.is_negative() {
                     sign = -sign;
-                    unsafe { SourceDigit::try_from(-value).unwrap_unchecked() }
+                    unsafe { Source::try_from(-value).unwrap_unchecked() }
                 } else {
-                    unsafe { SourceDigit::try_from(value).unwrap_unchecked() }
+                    unsafe { Source::try_from(value).unwrap_unchecked() }
                 }
             } else {
                 value
             };
-            let mut digits = Vec::<TargetDigit>::new();
-            let digit_mask = to_digit_mask::<SourceDigit>(SHIFT);
+            let mut digits = Vec::<Digit>::new();
+            let digit_mask = to_digit_mask::<Source>(SHIFT);
             while !value.is_zero() {
-                digits
-                    .push(unsafe { TargetDigit::try_from(value & digit_mask).unwrap_unchecked() });
+                digits.push(unsafe { Digit::try_from(value & digit_mask).unwrap_unchecked() });
                 value >>= SHIFT;
             }
             Self { sign, digits }
