@@ -26,6 +26,7 @@ pub trait AdditiveDigit = AssigningAdditiveMonoid
 pub trait BinaryDigit = AssigningAdditiveMonoid
     + AssigningBitwiseConjunctiveMagma
     + AssigningBitwiseDisjunctiveMonoid
+    + AssigningBitwiseExclusiveDisjunctiveMonoid
     + AssigningMultiplicativeMonoid
     + AssigningShiftingLeftMonoid<usize>
     + AssigningShiftingRightMonoid<usize>
@@ -134,13 +135,9 @@ where
 
 pub trait OppositiveDigit = ZeroableDigit;
 
-pub trait RightShiftableDigit = AdditiveDigit
-    + AssigningBitwiseExclusiveDisjunctiveMonoid
-    + Debug
-    + DivisibleDigit
-    + ShiftingRightMonoid
-    + TryFrom<usize>
-where DoublePrecisionOf<Self>: AssigningShiftingLeftMonoid<Self>;
+pub trait RightShiftableDigit =
+    AdditiveDigit + Debug + DivisibleDigit + ShiftingRightMonoid + TryFrom<usize>
+    where DoublePrecisionOf<Self>: AssigningShiftingLeftMonoid<Self>;
 
 pub trait UnitaryDigit = Unitary + OppositiveDigit;
 
@@ -149,6 +146,34 @@ pub trait ZeroableDigit = Oppose + Zeroable;
 pub(crate) type Sign = i8;
 pub(crate) type WindowDigit = u8;
 
+pub enum CheckedDivApproximationError {
+    TooLarge,
+    ZeroDivision,
+}
+
+impl CheckedDivApproximationError {
+    fn description(&self) -> &str {
+        match self {
+            CheckedDivApproximationError::TooLarge => {
+                "Division result too large to be expressed as floating point."
+            }
+            CheckedDivApproximationError::ZeroDivision => "Division by zero is undefined.",
+        }
+    }
+}
+
+impl Debug for CheckedDivApproximationError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        formatter.write_str(self.description())
+    }
+}
+
+impl Display for CheckedDivApproximationError {
+    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
+        Display::fmt(&self.description(), formatter)
+    }
+}
+
 pub(crate) fn binary_digits_to_base<
     SourceDigit: BinaryDigitConvertibleTo<TargetDigit>,
     TargetDigit,
@@ -336,32 +361,106 @@ pub(crate) fn binary_digits_to_lesser_binary_base<
     result
 }
 
-pub enum CheckedDivApproximationError {
-    TooLarge,
-    ZeroDivision,
+pub(crate) fn bitwise_and<Digit: BinaryDigit, const SHIFT: usize>(
+    mut longest: Vec<Digit>,
+    longest_sign: Sign,
+    mut shortest: Vec<Digit>,
+    shortest_sign: Sign,
+) -> (Sign, Vec<Digit>) {
+    if longest_sign.is_negative() {
+        longest = complement::<Digit, SHIFT>(&longest);
+    };
+    if shortest_sign.is_negative() {
+        shortest = complement::<Digit, SHIFT>(&shortest);
+    };
+    let result_size = if shortest_sign.is_negative() {
+        longest.len()
+    } else {
+        shortest.len()
+    };
+    let mut result = Vec::<Digit>::with_capacity(result_size);
+    for index in 0..shortest.len() {
+        result.push(longest[index] & shortest[index]);
+    }
+    for index in shortest.len()..result_size {
+        result.push(longest[index]);
+    }
+    let mut sign = longest_sign & shortest_sign;
+    if sign.is_negative() {
+        result.push(to_digit_mask::<Digit>(SHIFT));
+        result = complement::<Digit, SHIFT>(&result);
+    }
+    trim_leading_zeros(&mut result);
+    sign *= (result.len() > 1 || !result[0].is_zero()) as Sign;
+    (sign, result)
 }
 
-impl CheckedDivApproximationError {
-    fn description(&self) -> &str {
-        match self {
-            CheckedDivApproximationError::TooLarge => {
-                "Division result too large to be expressed as floating point."
-            }
-            CheckedDivApproximationError::ZeroDivision => "Division by zero is undefined.",
+pub(crate) fn bitwise_or<Digit: BinaryDigit, const SHIFT: usize>(
+    mut longest: Vec<Digit>,
+    longest_sign: Sign,
+    mut shortest: Vec<Digit>,
+    shortest_sign: Sign,
+) -> (Sign, Vec<Digit>) {
+    if longest_sign.is_negative() {
+        longest = complement::<Digit, SHIFT>(&longest);
+    };
+    if shortest_sign.is_negative() {
+        shortest = complement::<Digit, SHIFT>(&shortest);
+    };
+    let result_size = if shortest_sign.is_negative() {
+        shortest.len()
+    } else {
+        longest.len()
+    };
+    let mut result = Vec::<Digit>::with_capacity(result_size);
+    for index in 0..shortest.len() {
+        result.push(longest[index] | shortest[index]);
+    }
+    for index in shortest.len()..result_size {
+        result.push(longest[index]);
+    }
+    let sign = longest_sign | shortest_sign;
+    if sign.is_negative() {
+        result.push(to_digit_mask::<Digit>(SHIFT));
+        result = complement::<Digit, SHIFT>(&result);
+    }
+    trim_leading_zeros(&mut result);
+    (sign, result)
+}
+
+pub(crate) fn bitwise_xor<Digit: BinaryDigit, const SHIFT: usize>(
+    mut longest: Vec<Digit>,
+    longest_sign: Sign,
+    mut shortest: Vec<Digit>,
+    shortest_sign: Sign,
+) -> (Sign, Vec<Digit>) {
+    if longest_sign.is_negative() {
+        longest = complement::<Digit, SHIFT>(&longest);
+    };
+    if shortest_sign.is_negative() {
+        shortest = complement::<Digit, SHIFT>(&shortest);
+    };
+    let mut result = Vec::<Digit>::with_capacity(longest.len());
+    for index in 0..shortest.len() {
+        result.push(longest[index] ^ shortest[index]);
+    }
+    if shortest_sign.is_negative() {
+        let digit_mask = to_digit_mask::<Digit>(SHIFT);
+        for index in shortest.len()..longest.len() {
+            result.push(longest[index] ^ digit_mask);
         }
+    } else {
+        for index in shortest.len()..longest.len() {
+            result.push(longest[index]);
+        }
+    };
+    let sign = longest_sign ^ shortest_sign;
+    if sign.is_negative() {
+        result.push(to_digit_mask::<Digit>(SHIFT));
+        result = complement::<Digit, SHIFT>(&result);
     }
-}
-
-impl Debug for CheckedDivApproximationError {
-    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
-        formatter.write_str(self.description())
-    }
-}
-
-impl Display for CheckedDivApproximationError {
-    fn fmt(&self, formatter: &mut Formatter<'_>) -> std::fmt::Result {
-        Display::fmt(&self.description(), formatter)
-    }
+    trim_leading_zeros(&mut result);
+    (sign, result)
 }
 
 pub(crate) fn checked_div_approximation<
@@ -623,6 +722,19 @@ pub(crate) fn checked_rem_euclid<Digit: EuclidDivisibleDigit, const SHIFT: usize
     }
 }
 
+pub(crate) fn complement<Digit: BinaryDigit, const SHIFT: usize>(digits: &[Digit]) -> Vec<Digit> {
+    let mut result = Vec::<Digit>::with_capacity(digits.len());
+    let mut accumulator = Digit::one();
+    let digit_mask = to_digit_mask::<Digit>(SHIFT);
+    for index in 0..digits.len() {
+        accumulator += digits[index] ^ digit_mask;
+        result.push(accumulator & digit_mask);
+        accumulator >>= SHIFT;
+    }
+    debug_assert!(accumulator.is_zero());
+    result
+}
+
 pub(crate) fn digits_from_finite_positive_improper_float<
     Digit: Copy + ZeroableDigit,
     Value: Float,
