@@ -19,9 +19,9 @@ use pyo3::{ffi, AsPyPointer, Py, PyAny, PyErr, PyNativeType, PyRef, ToPyObject};
 use pyo3::{IntoPy, PyNumberProtocol, PyObject};
 
 use crate::traits::{
-    Abs, CheckedDiv, CheckedDivEuclid, CheckedDivRemEuclid, CheckedPow, CheckedPowRemEuclid,
-    CheckedRemEuclid, CheckedShl, CheckedShr, FromStrRadix, Gcd, Oppositive, Pow, Unitary,
-    Zeroable,
+    Abs, BitLength, CheckedDiv, CheckedDivEuclid, CheckedDivRemEuclid, CheckedPow,
+    CheckedPowRemEuclid, CheckedRemEuclid, CheckedShl, CheckedShr, FromStrRadix, Gcd, Oppositive,
+    Pow, Unitary, Zeroable,
 };
 
 pub mod big_int;
@@ -114,6 +114,10 @@ impl PyInt {
         }
     }
 
+    fn bit_length(&self) -> PyInt {
+        PyInt(self.0.clone().bit_length())
+    }
+
     #[pyo3(text_signature = "($self, other, /)")]
     fn gcd(&self, other: Self) -> PyInt {
         PyInt(self.0.clone().gcd(other.0))
@@ -232,6 +236,10 @@ impl PyNumberProtocol for PyInt {
         PyInt(lhs.0 + rhs.0)
     }
 
+    fn __and__(lhs: PyInt, rhs: PyInt) -> PyInt {
+        PyInt(lhs.0 & rhs.0)
+    }
+
     fn __divmod__(lhs: PyInt, rhs: PyInt) -> PyResult<(PyInt, PyInt)> {
         match lhs.0.checked_div_rem_euclid(rhs.0) {
             Some((quotient, remainder)) => Ok((PyInt(quotient), PyInt(remainder))),
@@ -292,6 +300,10 @@ impl PyNumberProtocol for PyInt {
         PyInt(-self.0.clone())
     }
 
+    fn __or__(lhs: PyInt, rhs: PyInt) -> PyInt {
+        PyInt(lhs.0 | rhs.0)
+    }
+
     fn __pow__(lhs: PyInt, rhs: PyInt, divisor: Option<PyInt>) -> PyResult<PyObject> {
         match divisor {
             Some(value) => {
@@ -346,6 +358,10 @@ impl PyNumberProtocol for PyInt {
             )),
         }
     }
+
+    fn __xor__(lhs: PyInt, rhs: PyInt) -> PyInt {
+        PyInt(lhs.0 ^ rhs.0)
+    }
 }
 
 fn hash(value: &_BigInt) -> usize {
