@@ -96,6 +96,27 @@ macro_rules! plain_abs_impl {
 
 plain_abs_impl!(f32 f64 i8 i16 i32 i64 i128 isize);
 
+pub trait BitLength {
+    type Output;
+
+    fn bit_length(self) -> Self::Output;
+}
+
+macro_rules! plain_bit_length_impl {
+    ($($t:ty)*) => ($(
+        impl BitLength for $t {
+            type Output = usize;
+
+            #[inline(always)]
+            fn bit_length(self) -> Self::Output {
+                (<$t>::BITS as usize) - (self.leading_zeros() as usize)
+            }
+        }
+    )*)
+}
+
+plain_bit_length_impl!(i8 i16 i32 i64 i128 isize u8 u16 u32 u64 u128 usize);
+
 pub trait DivEuclid<Divisor = Self> {
     type Output;
 
@@ -824,8 +845,12 @@ macro_rules! plain_gcd_impl {
 
             #[inline(always)]
             fn gcd(self, other: Self) -> Self::Output {
-                use crate::utils;
-                utils::gcd::<$t>(self, other)
+                let mut first = self;
+                let mut second = other;
+                while !second.is_zero() {
+                    (first, second) = (second, first.rem_euclid(second));
+                }
+                first
             }
         }
     )*)
